<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    <title>${task.name | n} has recovered from failure</title>
  </head>
  <body>
    ${task.name | n} has recovered from failure.
    <br/>
    The command <code style="background-color: #f0f0f0">${' '.join(subproc.command)}</code> has returned with code ${subproc.returncode}.
  </body>
</html>
