<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    <title>!!! ${task_name} DELAYED - ${current_sec}</title>
  </head>
  <body>
    The following process(es) are still running and policy is DELAY:
    % for task in running:
    <h4 style="background-color: #f0f0f0">Started for: ${task.formatted_sec}</h4>
    % if task.stdout_lines:
    <h4 style="border-bottom: 1px solid black">STDOUT</h4>
    <pre>${task.stdout_lines}</pre>
    % endif
    % if task.stderr_lines:
    <h4 style="border-bottom: 1px solid black">STDERR</h4>
    <pre>${task.stderr_lines}</pre>
    % endif
    % endfor
    <h4>Tasks waiting in the queue:</h4>
    <ol>
      % for task_sec in delay_queue:
      <li>${task_sec}</li>
      % endfor
    </ol>
  </body>
</html>
