<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    <title>${subproc.task_name} COMPLETED - ${subproc.formatted_sec}</title>
  </head>
  <body>
    The command <code style="background-color: #f0f0f0">${' '.join(subproc.command)}</code> has returned with code ${subproc.returncode}.
    % if subproc.stdout_lines:
    <h4 style="border-bottom: 1px solid black">STDOUT</h4>
    <pre>${subproc.stdout_lines}</pre>
    % endif
    % if subproc.stderr_lines:
    <h4 style="border-bottom: 1px solid black">STDERR</h4>
    <pre>${subproc.stderr_lines}</pre>
    % endif
  </body>
</html>
